import * as process from 'process';
import cli from '../cli';
import * as yargs from 'yargs';
import { SSHService } from '../session/ssh';
import { getLocalExternalIp, scanNetwork } from '../utils/device';
import chalk from 'chalk';
import shelljs = require('shelljs');

exports.command = 'devices';
exports.desc = 'Discover devices on your local network';

exports.handler = async (argv: yargs.Argv) => {
    let devicesIP: string[] | void;
    cli.stream.print('Scanning network:');

    const ip: string[] = getLocalExternalIp().split('.');
    ip[ip.length - 1] = '255';
    const broadcastAddress = ip.join('.');
    try {
        await shelljs.exec(`ping ${broadcastAddress} -c 1`, { async: true, silent: true });
    } catch (e) {
        cli.stream.stop();
        cli.stream.error(`Broadcasting ping failed on ${broadcastAddress}, all devices might not appear.`);
    }

    devicesIP = await scanNetwork()
        .catch((e) => {
            cli.stream.stop();
            cli.stream.error('Error scanning network: ' + e.message);
            return;
        });
    if (devicesIP === undefined) {
        cli.stream.stop();
        cli.stream.error('No devices found');
        return process.exit();
    }

    const devicesIPAndHostname = await SSHService.fetchBootHostname(devicesIP)
        .catch((e) => {
            cli.stream.stop();
            cli.stream.error(e);
        });
    if (devicesIPAndHostname === undefined) {
        cli.stream.stop();
        cli.stream.error('Error fetching devices hostnames');
        return process.exit();
    }

    const devicesList = devicesIPAndHostname.map(({ ip, hostname, snipsHostname }) => {
        if (snipsHostname.length > 0) {
            return `- ${hostname} (${ip}, ${snipsHostname})`;
        }
        if (hostname.length === 0) {
            return `${ip}`;
        }
        return `- ${hostname} (${ip})`;
    });
    cli.stream.done();
    if (devicesList.length === 1) {
        cli.stream.print(`Found ${devicesList.length} device:`);
    } else {
        cli.stream.print(`Found ${devicesList.length} devices:`);
    }

    devicesList.forEach((device) => {
        cli.stream.print(device);
    });

    cli.stream.hint('You can now connect with ' + chalk.blue('sam connect <hostname>'));
    process.exit();
};
